from __future__ import absolute_import
import wx
from wx.lib.pubsub import pub
from matplotlib import cm
import numpy as np
from .ztv_wx_lib import force_textctrl_color_update, set_textctrl_background_color, validate_textctrl_str
import sys

class ColorPanel(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize)
        self.ztv_frame = self.GetTopLevelParent()
        self.eventID_to_cmap = {wx.NewId(): x for x in self.ztv_frame.available_cmaps}
        self.cmap_to_eventID = {self.eventID_to_cmap[x]: x for x in self.eventID_to_cmap}
        self.last_string_values = {'minval':'', 'maxval':''}

        v_sizer1 = wx.BoxSizer(wx.VERTICAL)
        v_sizer1.AddSpacer((0, 0), 1, wx.EXPAND)

        values_sizer = wx.FlexGridSizer( 3, 5, 0, 0 )
        values_sizer.SetFlexibleDirection( wx.BOTH )
        values_sizer.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.minval_static_text = wx.StaticText( self, wx.ID_ANY, u"Min", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.minval_static_text.Wrap( -1 )
        values_sizer.Add(self.minval_static_text, 0, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 0)

        textentry_font = wx.Font(14, wx.FONTFAMILY_MODERN, wx.NORMAL, wx.FONTWEIGHT_LIGHT, False)

        self.minval_textctrl = wx.TextCtrl(self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize,
                                           wx.TE_PROCESS_ENTER)
        self.minval_textctrl.SetFont(textentry_font)
        values_sizer.Add(self.minval_textctrl, 0, wx.ALL, 2)
        self.minval_textctrl.Bind(wx.EVT_TEXT, self.minval_textctrl_changed)
        self.minval_textctrl.Bind(wx.EVT_TEXT_ENTER, self.minval_textctrl_entered)

        self.set_min_button = wx.Button(self, wx.ID_ANY, u"Min", wx.DefaultPosition, wx.DefaultSize, 0)
        values_sizer.Add(self.set_min_button, 0, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 2)
        self.set_min_button.Bind(wx.EVT_BUTTON, self.on_set_min_button)
        values_sizer.AddSpacer((0,0), 0, wx.EXPAND)
        temp_list = self.ztv_frame.available_value_modes_on_new_image[:]
        temp_list[temp_list.index('data-min/max')] = 'min'
        self.choose_min_value_mode_on_new_image = wx.Choice(self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize,
                                                            temp_list, 0)
        self.choose_min_value_mode_on_new_image.SetSelection(0)
        values_sizer.Add(self.choose_min_value_mode_on_new_image, 0,
                         wx.ALL|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 2)
        self.Bind(wx.EVT_CHOICE, self.on_choose_min_value_mode_on_new_image, self.choose_min_value_mode_on_new_image)

        values_sizer.AddSpacer((0,0), 0, wx.EXPAND)

        self.auto_set_minmax_button = wx.Button(self, wx.ID_ANY, u"Auto", wx.DefaultPosition, wx.DefaultSize, 0)
        values_sizer.Add(self.auto_set_minmax_button, 0, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 2)
        self.auto_set_minmax_button.Bind(wx.EVT_BUTTON, self.on_auto_set_minmax_button)

        self.set_minmax_button = wx.Button(self, wx.ID_ANY, u"Min/Max", wx.DefaultPosition, wx.DefaultSize, 0)
        values_sizer.Add(self.set_minmax_button, 0, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 2)
        self.set_minmax_button.Bind(wx.EVT_BUTTON, self.on_set_minmax_button)

        self.minval_static_text = wx.StaticText(self, wx.ID_ANY, u"On load image:",
                                                wx.DefaultPosition, wx.DefaultSize, 0 )
        self.minval_static_text.Wrap( -1 )
        values_sizer.Add(self.minval_static_text, 0, wx.ALL, 2)
        temp_list = self.ztv_frame.available_value_modes_on_new_image[:]
        temp_list[temp_list.index('data-min/max')] = 'min/max'
        temp_list.insert(0, '-'),
        self.choose_minmax_value_mode_on_new_image = wx.Choice(self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize,
                                                               temp_list, 0)
        self.choose_minmax_value_mode_on_new_image.SetSelection(1)
        values_sizer.Add(self.choose_minmax_value_mode_on_new_image, 0,
                         wx.ALL|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 2)
        self.Bind(wx.EVT_CHOICE, self.on_choose_minmax_value_mode_on_new_image,
                  self.choose_minmax_value_mode_on_new_image)

        maxval_static_text = wx.StaticText( self, wx.ID_ANY, u"Max", wx.DefaultPosition, wx.DefaultSize, 0 )
        maxval_static_text.Wrap( -1 )
        values_sizer.Add(maxval_static_text, 0, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 2)

        self.maxval_textctrl = wx.TextCtrl(self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize,
                                           wx.TE_PROCESS_ENTER)
        self.maxval_textctrl.SetFont(textentry_font)
        values_sizer.Add(self.maxval_textctrl, 0, wx.ALL, 2)
        self.maxval_textctrl.Bind(wx.EVT_TEXT, self.maxval_textctrl_changed)
        self.maxval_textctrl.Bind(wx.EVT_TEXT_ENTER, self.maxval_textctrl_entered)

        self.set_max_button = wx.Button(self, wx.ID_ANY, u"Max", wx.DefaultPosition, wx.DefaultSize, 0)
        values_sizer.Add(self.set_max_button, 0, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 2)
        self.set_max_button.Bind(wx.EVT_BUTTON, self.on_set_max_button)
        values_sizer.AddSpacer((0,0), 0, wx.EXPAND)
        temp_list = self.ztv_frame.available_value_modes_on_new_image[:]
        temp_list[temp_list.index('data-min/max')] = 'max'
        self.choose_max_value_mode_on_new_image = wx.Choice(self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize,
                                                            temp_list, 0)
        self.choose_max_value_mode_on_new_image.SetSelection(0)
        values_sizer.Add(self.choose_max_value_mode_on_new_image, 0,
                         wx.ALL|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 2)
        self.Bind(wx.EVT_CHOICE, self.on_choose_max_value_mode_on_new_image, self.choose_max_value_mode_on_new_image)

        v_sizer1.Add(values_sizer, 0)
        v_sizer1.AddSpacer((0, 6), 0, 0)
        v_sizer1.Add(wx.StaticLine(self), flag=wx.EXPAND)
        v_sizer1.AddSpacer((0, 6), 0, 0)
        cmap_sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.init_cmap_popup_menu()
        self.cmap_button = wx.Button(self, wx.ID_ANY, 'X'*max([len(a) for a in self.ztv_frame.available_cmaps]),
                                     wx.DefaultPosition, wx.DefaultSize, 0)
        if hasattr(self.cmap_button, 'SetBitmap'):
            self.cmap_button.SetBitmap(self.cmap_button_bitmaps[self.ztv_frame.cmap])
        cmap_sizer.Add(self.cmap_button, 0, wx.ALL|wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL, 2)
        self.cmap_button.Bind(wx.EVT_LEFT_DOWN, self.on_cmap_button)
        cmap_options_sizer = wx.BoxSizer(wx.VERTICAL)
        self.is_cmap_inverted_checkbox = wx.CheckBox(self, -1, 'inverted', wx.DefaultPosition, wx.DefaultSize, 0)
        cmap_options_sizer.Add(self.is_cmap_inverted_checkbox, 0)
        self.Bind(wx.EVT_CHECKBOX, self.on_is_cmap_inverted_checkbox, self.is_cmap_inverted_checkbox)
        self.choose_scaling = wx.Choice(self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize,
                                        self.ztv_frame.available_scalings, 0)
        self.choose_scaling.SetSelection(0)
        scaling_sizer = wx.BoxSizer(wx.HORIZONTAL)
        scaling_sizer.Add(wx.StaticText( self, wx.ID_ANY, u"Scaling", wx.DefaultPosition, wx.DefaultSize, 0 ), 0)
        scaling_sizer.Add(self.choose_scaling, 0)
        cmap_options_sizer.Add(scaling_sizer, 0)
        self.Bind(wx.EVT_CHOICE, self.on_choose_scaling, self.choose_scaling)
        cmap_sizer.Add(cmap_options_sizer, 0)
        v_sizer1.Add(cmap_sizer, 0)
        v_sizer1.AddSpacer((0, 0), 1, wx.EXPAND)
        self.SetSizer(v_sizer1)
        pub.subscribe(self.on_clim_changed, 'clim-changed')   
        pub.subscribe(self.on_cmap_changed, 'cmap-changed')
        pub.subscribe(self.on_is_cmap_inverted_changed, 'is-cmap-inverted-changed')
        pub.subscribe(self.on_scaling_changed, 'scaling-changed')
#         self.Bind(wx.EVT_NAVIGATION_KEY, self.on_navigation_key)

    def on_choose_scaling(self, evt):
        wx.CallAfter(pub.sendMessage, 'set-scaling', msg=(self.ztv_frame._pause_redraw_image, evt.GetString()))

    def init_cmap_popup_menu(self):
        cmap_button_bitmap_height = 10
        cmap_button_bitmap_width = 200
        cmap_menu_bitmap_height = 20
        cmap_menu_bitmap_width = 200
        self.cmap_button_bitmaps = {}
        self.cmap_menu_bitmaps = {}
        for cmap in self.ztv_frame.available_cmaps:
            temp = cm.ScalarMappable(cmap=cmap)
            rgba = temp.to_rgba(np.outer(np.ones(cmap_button_bitmap_height, dtype=np.uint8),
                                         np.arange(cmap_button_bitmap_width, dtype=np.uint8)))
            self.cmap_button_bitmaps[cmap] = wx.BitmapFromBufferRGBA(cmap_button_bitmap_width, cmap_button_bitmap_height,
                                                                     np.uint8(np.round(rgba*255)))
            rgba = temp.to_rgba(np.outer(np.ones(cmap_menu_bitmap_height, dtype=np.uint8),
                                         np.arange(cmap_menu_bitmap_width, dtype=np.uint8)))
            self.cmap_menu_bitmaps[cmap] = wx.BitmapFromBufferRGBA(cmap_menu_bitmap_width, cmap_menu_bitmap_height,
                                                                   np.uint8(np.round(rgba*255)))
        menu = wx.Menu()
        for cmap in self.ztv_frame.available_cmaps:
            menu_item = menu.AppendCheckItem(self.cmap_to_eventID[cmap], cmap)
            wx.EVT_MENU(menu, self.cmap_to_eventID[cmap], self.on_change_cmap_event)
            if hasattr(menu_item, 'SetBitmap'):
                menu_item.SetBitmap(self.cmap_menu_bitmaps[cmap])
        self.cmap_popup_menu = menu

    def on_change_cmap_event(self, event):
        wx.CallAfter(pub.sendMessage, 'set-cmap', msg=(self.ztv_frame._pause_redraw_image,
                                                       self.eventID_to_cmap[event.GetId()]))

    def on_is_cmap_inverted_checkbox(self, evt):
        wx.CallAfter(pub.sendMessage, 'set-cmap-inverted', msg=(self.ztv_frame._pause_redraw_image, evt.IsChecked()))

    def on_is_cmap_inverted_changed(self, msg=None):
        self.is_cmap_inverted_checkbox.SetValue(self.ztv_frame.is_cmap_inverted)

    def on_scaling_changed(self, msg=None):
        self.choose_scaling.SetSelection(self.ztv_frame.available_scalings.index(self.ztv_frame.scaling))

    def on_cmap_button(self, evt):
        for cmap in self.ztv_frame.available_cmaps:
            self.cmap_popup_menu.Check(self.cmap_to_eventID[cmap], False)
        self.cmap_popup_menu.Check(self.cmap_to_eventID[self.ztv_frame.cmap], True)
        pos = self.ScreenToClient(wx.GetMousePosition())
        self.PopupMenu(self.cmap_popup_menu, pos)

    def on_choose_min_value_mode_on_new_image(self, evt):
        new_val = [a for a in self.ztv_frame.available_value_modes_on_new_image if
                   evt.GetString() in a][0]
        self.ztv_frame.min_value_mode_on_new_image = new_val
        if (self.choose_min_value_mode_on_new_image.GetSelection() ==
            self.choose_max_value_mode_on_new_image.GetSelection()):
            self.choose_minmax_value_mode_on_new_image.SetSelection( \
                        self.choose_min_value_mode_on_new_image.GetSelection() + 1)
        else:
            self.choose_minmax_value_mode_on_new_image.SetSelection(0)

    def on_choose_max_value_mode_on_new_image(self, evt):
        new_val = [a for a in self.ztv_frame.available_value_modes_on_new_image if
                   evt.GetString() in a][0]
        self.ztv_frame.max_value_mode_on_new_image = new_val
        if (self.choose_min_value_mode_on_new_image.GetSelection() ==
            self.choose_max_value_mode_on_new_image.GetSelection()):
            self.choose_minmax_value_mode_on_new_image.SetSelection( \
                        self.choose_min_value_mode_on_new_image.GetSelection() + 1)
        else:
            self.choose_minmax_value_mode_on_new_image.SetSelection(0)

    def on_choose_minmax_value_mode_on_new_image(self, evt):
        if evt.GetString != '':
            new_val = [a for a in self.ztv_frame.available_value_modes_on_new_image if
                       evt.GetString() in a][0]
            self.ztv_frame.min_value_mode_on_new_image = new_val
            self.ztv_frame.max_value_mode_on_new_image = new_val
            new_index = self.ztv_frame.available_value_modes_on_new_image.index(new_val)
            self.choose_min_value_mode_on_new_image.SetSelection(new_index)
            self.choose_max_value_mode_on_new_image.SetSelection(new_index)

    def on_auto_set_minmax_button(self, evt):
        self.ztv_frame.set_clim_to_auto()

    def on_set_minmax_button(self, evt):
        self.ztv_frame.set_clim_to_minmax()

    def on_set_min_button(self, evt):
        self.ztv_frame.set_clim((self.ztv_frame._pause_redraw_image, [self.ztv_frame.display_image.min(), None]))
        if self.FindFocus() == self.minval_textctrl:
            self.minval_textctrl.SetSelection(-1, -1)

    def on_set_max_button(self, evt):
        self.ztv_frame.set_clim((self.ztv_frame._pause_redraw_image, [None, self.ztv_frame.display_image.max()]))
        if self.FindFocus() == self.maxval_textctrl:
            self.maxval_textctrl.SetSelection(-1, -1)

    def on_clim_changed(self, msg=None):
        new_minval_str = "{: .9g}".format(self.ztv_frame.clim[0])
        new_maxval_str = "{: .9g}".format(self.ztv_frame.clim[1])
        if new_minval_str != self.last_string_values['minval']:
            self.minval_textctrl.SetValue(new_minval_str)
            set_textctrl_background_color(self.minval_textctrl, 'ok')
            self.last_string_values['minval'] = new_minval_str
            force_textctrl_color_update(self.minval_textctrl)
        if new_maxval_str != self.last_string_values['maxval']:
            self.maxval_textctrl.SetValue(new_maxval_str)
            set_textctrl_background_color(self.maxval_textctrl, 'ok')
            self.last_string_values['maxval'] = new_maxval_str
            force_textctrl_color_update(self.maxval_textctrl)

    def on_cmap_changed(self, msg=None):
        if hasattr(self.cmap_button, 'SetBitmap'):
            self.cmap_button.SetBitmap(self.cmap_button_bitmaps[self.ztv_frame.cmap])
        self.cmap_button.SetLabel(self.ztv_frame.cmap)

    def minval_textctrl_changed(self, evt):
        validate_textctrl_str(self.minval_textctrl, float, self.last_string_values['minval'])

    def minval_textctrl_entered(self, evt):
        if validate_textctrl_str(self.minval_textctrl, float, self.last_string_values['minval']):
            self.last_string_values['minval'] = self.minval_textctrl.GetValue()
            self.ztv_frame.set_clim((self.ztv_frame._pause_redraw_image,
                                     [float(self.minval_textctrl.GetValue()), None]))
            self.minval_textctrl.SetSelection(-1, -1)

    def maxval_textctrl_changed(self, evt):
        validate_textctrl_str(self.maxval_textctrl, float, self.last_string_values['maxval'])

    def maxval_textctrl_entered(self, evt):
        if validate_textctrl_str(self.maxval_textctrl, float, self.last_string_values['maxval']):
            self.last_string_values['maxval'] = self.maxval_textctrl.GetValue()
            self.ztv_frame.set_clim((self.ztv_frame._pause_redraw_image, 
                                     [None, float(self.maxval_textctrl.GetValue())]))
            self.maxval_textctrl.SetSelection(-1, -1)





from __future__ import absolute_import
import wx
import os
import glob
import sys
import subprocess
import pdb
from .ztv_wx_lib import set_textctrl_background_color

# TODO:  fix tab order through controls
# TODO:  fix alignment in text field, ideally would be right aligned, but may not be able to do that on OSX
# TODO: limit default allowed_extensions to .fits/.fits.gz  (rather than .fits/.gz)
# TODO: fix that is currently hardwired to .fits/.gz file endings and allow calling code to specify other options

# TODO: implement a None option? or that None is italixiszed in text box and dropdown menu as a special thing?

class Error(Exception):
    pass


class FilePicker(wx.Panel):
    def __init__(self, parent,
                 is_files_not_dirs=True,  # True to look for files, False to look for dirs
                 max_history_items=20,
                 history=None,  # pre-load items into history
                 sticky_history=None,  # force one or more history items to be 'sticky' and remain at the top of the list
                 allowed_extensions="FITS files (.fits)|.fits|gzipped files (.gz)|.gz", # ignored in the dir case
                 title=None,  # None leads to a default, for NO title, use:  ''
                 assumed_prefix=None, #  can add a prefix to the file/dir name that will not be shown
                                      # assumed_prefix can be changed with:   self.set_assumed_prefix
                                      # Note that if an entry starts with /, then abspath is assumed and assumed_prefix is ignored
                                      # Basically, the interface just "hides" assumed_prefix from the user in the GUI
                 allow_glob_matching=False,  # if true, then entries can be, e.g.:   image*_[0-9][0-9][0-9].fits
                                             # allow_glob_matching is NOT compatible with is_files_not_dirs=False
                 default_entry=None,
                 maintain_default_entry_in_recents=False,  # can be True, or an integer to indicate where to put in list (e.g. 0, 1, -1)
                 ):
        wx.Panel.__init__(self, parent, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL)
        self.is_files_not_dirs = is_files_not_dirs
        self.max_history_items = max_history_items
        if history is None:
            self.history = []
        else:
            self.history = [a + '/' if (os.path.isdir(a) and not a.endswith('/')) else a for a in history]
        if sticky_history is None:
            self.sticky_history = []
        else:
            self.sticky_history = [a + '/' if (os.path.isdir(a) and not a.endswith('/')) else a for a in sticky_history]
        self.allowed_extensions = allowed_extensions
        if title is None:
            if self.is_files_not_dirs:
                title = 'File:'
            else:
                title = 'Dir:'
        self.assumed_prefix = assumed_prefix
        self.allow_glob_matching = allow_glob_matching
        if self.allow_glob_matching and not self.is_files_not_dirs:
            raise Error("allow_glob_matching=True and is_files_not_dirs=False is not compatible")
        if default_entry is None:
            default_entry = os.path.expanduser('~') + '/'
        self.default_entry = default_entry
        self.maintain_default_entry_in_recents = maintain_default_entry_in_recents
        self.title = title
        self.last_valid_entry = ''
        self.reset_auto_completion_info()
        self.current_textctrl_mode_is_ok = True
        self.pause_on_current_textctrl_changed = False

        h_sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.recent_choice = wx.Choice(self, -1, size=(24, -1), choices=self.get_history())
        self.Bind(wx.EVT_CHOICE, self.on_recent_choice, self.recent_choice)
        textentry_font = wx.Font(14, wx.FONTFAMILY_MODERN, wx.NORMAL, wx.FONTWEIGHT_LIGHT, False)
        self.current_textctrl = wx.TextCtrl(self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize,
                                            wx.TE_PROCESS_ENTER | wx.TE_RIGHT)
        # note that wx.TE_RIGHT has no effect on Mac OSX, but is worthwhile leaving in for other systems
        self.current_textctrl.SetFont(textentry_font)
        self.current_textctrl.Bind(wx.EVT_TEXT, self.on_current_textctrl_changed)
        self.current_textctrl.Bind(wx.EVT_TEXT_ENTER, self.on_current_textctrl_entered)
        self.current_textctrl.Bind(wx.EVT_CHAR, self.on_key_press_textctrl, self.current_textctrl)
        h_sizer.Add(wx.StaticText(self, -1, title), 0, wx.ALIGN_CENTER_VERTICAL)
        h_sizer.Add(self.current_textctrl, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
        h_sizer.AddSpacer((2, 0), 0)
        h_sizer.Add(self.recent_choice, 0, wx.ALIGN_CENTER_VERTICAL)
        h_sizer.AddSpacer((2, 0), 0)
        self.browser_button = wx.Button(self, -1, '...', style=wx.BU_EXACTFIT)
        self.Bind(wx.EVT_BUTTON, self.on_browser_button, self.browser_button)
        h_sizer.Add(self.browser_button, 0, wx.ALIGN_CENTER_VERTICAL)
        self.SetSizer(h_sizer)
        self.current_textctrl_SetValue(self.default_entry)
        self._on_load(self.current_textctrl_GetValue())

    def on_load(self, new_entry):
        # this is really a placeholder method, to be replaced when this class is implemented with a method that actually does something
        pass
        
    def set_assumed_prefix(self, new_prefix):
        self.assumed_prefix = new_prefix
        self.update_recent_choice()

    def strip_assumed_prefix(self, entry):
        if self.assumed_prefix is not None and entry.startswith(self.assumed_prefix):
            return entry.replace(self.assumed_prefix, '', 1)
        else:
            return entry

    def addback_assumed_prefix(self, entry):
        if not entry.startswith('/') and not entry.startswith('~') and self.assumed_prefix is not None:
            return os.path.join(self.assumed_prefix, entry)
        else:
            return entry

    def current_textctrl_SetValue(self, entry):
        self.current_textctrl.SetValue(self.strip_assumed_prefix(entry))

    def current_textctrl_GetValue(self):
        return self.addback_assumed_prefix(self.current_textctrl.GetValue())

    def set_current_entry(self, new_entry):
        """
        This provides an external hook to call from external code to load an entry.
        It is not used internally.
        """
        self.current_textctrl_SetValue(new_entry)
        self.prepend_to_history(new_entry)
        cur_history = self.update_recent_choice()
        self.recent_choice.SetSelection(cur_history.index(new_entry))

    def update_recent_choice(self):
        self.recent_choice.Clear()
        cur_history = self.get_history()
        self.recent_choice.AppendItems(cur_history)
        return cur_history

    def prepend_to_history(self, new_entry):
        if new_entry in self.history:  # pop if necessary so we can reinsert at start of list
            self.history.pop(self.history.index(new_entry))
        self.history.insert(0, new_entry)
        self.update_recent_choice()

    def _on_load(self, new_entry):
        """
        this is a private method that just updates the internals and then calls self.on_load
        """
        self.last_valid_entry = new_entry
        if not self.is_files_not_dirs and not self.last_valid_entry.endswith('/'):
            self.last_valid_entry += '/'
        self.current_textctrl_SetValue(self.last_valid_entry)
        self.current_textctrl.SetSelection(-1, -1)
        self.prepend_to_history(self.last_valid_entry)
        self.update_recent_choice()
        self.on_load(self.last_valid_entry)
        set_textctrl_background_color(self.current_textctrl, 'ok')

    def get_history(self):
        """
        returns what the current history should be, taking in to account:
            self.history (the actual history, trimmed to self.max_history_items)
            self.sticky_history (items to stick at the top of the generated history)
            self.max_history (trim the output to this length)
            self.assumed_prefix (stripped off as necessary)
        """
        history = self.history[:]  # want a copy
        for cur_sticky in self.sticky_history[::-1]:
            if cur_sticky in history:
                history.pop(history.index(cur_sticky))
            history.insert(0, cur_sticky)
        history = history[:self.max_history_items]
        if self.maintain_default_entry_in_recents is not False:
            if type(self.maintain_default_entry_in_recents) is int:
                if self.default_entry in self.history:
                    self.history.pop(self.history.index(self.default_entry))
                self.history.insert(self.maintain_default_entry_in_recents, self.default_entry)
            else:
                if self.default_entry not in self.history:
                    self.history.insert(999999, self.default_entry)
        if self.assumed_prefix is None:
            return history[:self.max_history_items]
        else:
            return [self.strip_assumed_prefix(a) for a in history[:self.max_history_items]]

    def on_browser_button(self, evt):
        cur_entry = self.current_textctrl_GetValue()
        if len(cur_entry) == 0:
            cur_entry = self.default_entry
        cur_entry = os.path.abspath(os.path.expanduser(cur_entry))
        cur_entry_parts = [a for a in ['/'] + cur_entry.split('/') if len(a) > 0]
        while (len(cur_entry_parts) >= 1 and
               not (os.path.isdir(os.path.join(*cur_entry_parts)) or
                    os.path.isfile(os.path.join(*cur_entry_parts)))):
            cur_entry_parts = cur_entry_parts[:-1]
        if len(cur_entry_parts) > 1:
            cur_entry = os.path.join(*cur_entry_parts)
        else:
            cur_entry = os.path.expanduser('~')
        if os.path.isdir(cur_entry):
            cur_basename = ''
            cur_dirname = cur_entry
        else:
            cur_basename = os.path.basename(cur_entry)
            cur_dirname = os.path.dirname(cur_entry)
        if self.is_files_not_dirs:
            dlg = wx.FileDialog(self, "Choose a file:", style=wx.DD_DEFAULT_STYLE,
                                defaultDir=cur_dirname, defaultFile=cur_basename, wildcard=self.allowed_extensions)
            # TODO: defaultFile is not being set to cur_basename in the dialog box
        else:
            dlg = wx.DirDialog(self, "Choose a directory:", style=wx.DD_DEFAULT_STYLE, defaultPath=cur_dirname)
        if dlg.ShowModal() == wx.ID_OK:
            self._on_load(dlg.GetPath())
        dlg.Destroy()

    def reset_auto_completion_info(self):
        self.auto_completion_info = {'current_base':'', 'possible_completions':[],
                                     'current_completion':'', 'current_selection':(0, 0)}

    def validate_current_textctrl_value(self):
        is_entry_valid = False
        if (self.current_textctrl_GetValue() == self.default_entry):
            is_entry_valid = True   # catch for case of default_entry == '' and nothing in field
        new_entry = os.path.abspath(os.path.expanduser(self.current_textctrl_GetValue()))
        if self.is_files_not_dirs:
            if self.allow_glob_matching:
                if len(glob.glob(new_entry)) > 0:
                    is_entry_valid = True
            elif os.path.isfile(new_entry):
                    is_entry_valid = True
        else:
            if os.path.isdir(new_entry):
                is_entry_valid = True
        if new_entry == self.default_entry:
            is_entry_valid = True
        self.current_textctrl_mode_is_ok = False
        if is_entry_valid:
            if self.last_valid_entry == new_entry:
                set_textctrl_background_color(self.current_textctrl, 'ok')
                self.current_textctrl_mode_is_ok = True
            else:
                set_textctrl_background_color(self.current_textctrl, 'enter-needed', 
                                              'Press enter in this field to load')
            return True
        else:
            set_textctrl_background_color(self.current_textctrl, 'invalid', 'Not found on disk')
            return False

    def on_key_press_textctrl(self, evt):
        if evt.GetKeyCode() == wx.WXK_TAB:
            if self.current_textctrl_mode_is_ok:
                if evt.ShiftDown():
                    evt.Skip()
                else:
                    self.recent_choice.SetFocus()
            else:  # try auto completion
                new_entry = self.current_textctrl_GetValue()
                add_on = ''  # hack to add back the trailing / as needed
                if new_entry.endswith('/'):
                    add_on = '/'
                new_entry = (os.path.abspath(os.path.expanduser(new_entry)) + add_on).replace('//', '/')
                if ((not new_entry.startswith(self.auto_completion_info['current_base'])) or
                    (new_entry not in self.auto_completion_info['possible_completions']) or
                    (self.auto_completion_info['current_selection'] != self.current_textctrl.GetSelection())):
                    self.reset_auto_completion_info()
                if len(self.auto_completion_info['possible_completions']) == 0:
                    self.auto_completion_info['current_base'] = new_entry
                    possible_completions = glob.glob(new_entry + '*')
                    if self.is_files_not_dirs:
                        # TODO: un-hardwire the endswith fits/gz and connect as a parameter to the FileDialog filter parameter
                        if len(possible_completions) == 1 and os.path.isdir(possible_completions[0]):
                            self.auto_completion_info['current_base'] = possible_completions[0] + '/'
                            possible_completions = glob.glob(possible_completions[0] + '/*')
                        possible_completions = [a for a in possible_completions if
                                                (os.path.isdir(a) or
                                                 (os.path.isfile(a) and
                                                  (a.endswith(".fits") or a.endswith(".fits.gz"))))]
                    else:
                        possible_completions = [a for a in possible_completions if os.path.isdir(a)]
                        possible_completions = [a if a.endswith('/') else a + '/' for a in possible_completions]
                    self.auto_completion_info['possible_completions'] = possible_completions
                    if len(possible_completions) > 0:
                        self.auto_completion_info['current_completion'] = possible_completions[-1]
                    else:
                        self.reset_auto_completion_info()
                if len(self.auto_completion_info['possible_completions']) > 0:
                    if len(self.auto_completion_info['possible_completions']) == 1:
                        new_entry = self.auto_completion_info['possible_completions'][0]
                        self.current_textctrl_SetValue(new_entry)
                        self.current_textctrl.SetInsertionPoint(999)
                        self.reset_auto_completion_info()
                    else:
                        cur_index = self.auto_completion_info['possible_completions'].index(self.auto_completion_info['current_completion'])
                        step = 1
                        if evt.ShiftDown:
                            step = -1
                        cur_index = (cur_index + step) % len(self.auto_completion_info['possible_completions'])
                        new_entry = self.auto_completion_info['possible_completions'][cur_index]
                        self.auto_completion_info['current_completion'] = new_entry
                        s1 = len(self.auto_completion_info['current_base'])
                        s2 = len(self.auto_completion_info['current_completion'])
                        self.pause_on_current_textctrl_changed = True
                        if self.assumed_prefix is not None and new_entry.startswith(self.assumed_prefix):
                            s1 -= len(self.assumed_prefix)
                            s2 -= len(self.assumed_prefix)
                        self.auto_completion_info['current_selection'] = (s1, s2)
                        self.current_textctrl_SetValue(new_entry)
                        self.current_textctrl.SetSelection(s1, s2)
                        self.pause_on_current_textctrl_changed = False
                        self.validate_current_textctrl_value()
        else:
            evt.Skip()

    def on_current_textctrl_changed(self, evt):
        if not self.pause_on_current_textctrl_changed:
            new_entry = self.current_textctrl_GetValue()
            if ((not new_entry.startswith(self.auto_completion_info['current_base'])) or
                (new_entry not in self.auto_completion_info['possible_completions']) or
                (self.auto_completion_info['current_selection'] != self.current_textctrl.GetSelection())):
                self.reset_auto_completion_info()
            self.validate_current_textctrl_value()

    def on_current_textctrl_entered(self, evt):
        if self.validate_current_textctrl_value():
            self.reset_auto_completion_info()
            new_entry = self.current_textctrl_GetValue()
            if new_entry == self.default_entry:
                self._on_load(new_entry)
            else:
                self._on_load(os.path.abspath(new_entry))

    def on_recent_choice(self, event):
        new_choice = event.GetString()
        if not new_choice.startswith('/') and self.assumed_prefix is not None:
            new_choice = os.path.join(self.assumed_prefix, new_choice)
        self._on_load(new_choice)



class MyFrame(wx.Frame):
    def __init__(self):
        wx.Frame.__init__(self, None, wx.ID_ANY, title='Testcase')
        v_sizer = wx.BoxSizer(wx.VERTICAL)
        v_sizer.Add(wx.StaticText(self, -1, "Some misc information"), 0)
        v_sizer.AddSpacer((0, 20), 0)
        v_sizer.Add(FilePicker(self), 0, wx.EXPAND)
        v_sizer.AddSpacer((0, 0), 1, wx.EXPAND)
        sticky_history = [os.path.expanduser('~'), os.path.expanduser('~/Dropbox'), os.path.expanduser('~/Documents')]
        v_sizer.Add(wx.StaticLine(self, -1, style=wx.LI_HORIZONTAL), 0, wx.EXPAND|wx.ALL, 5)
        file_picker1 = FilePicker(self, allow_glob_matching=True, title='glob match')
        v_sizer.Add(file_picker1, 0, wx.EXPAND)

        history = sticky_history[:]
        v_sizer.Add(wx.StaticLine(self, -1, style=wx.LI_HORIZONTAL), 0, wx.EXPAND|wx.ALL, 5)
        self.dir_picker = FilePicker(self, is_files_not_dirs=False, history=history)
        self.file_picker = FilePicker(self, assumed_prefix=os.path.expanduser('~/'), allow_glob_matching=True, 
                                      title='glob match')
        self.dir_picker.on_load = self.file_picker.set_assumed_prefix
        self.file_picker.on_load = self.file_picker_on_load

        v_sizer.Add(self.dir_picker, 0, wx.EXPAND)
        v_sizer.AddSpacer((0, 10), 0, wx.EXPAND)
        v_sizer.Add(self.file_picker, 0, wx.EXPAND)
        v_sizer.Add(wx.StaticLine(self, -1, style=wx.LI_HORIZONTAL), 0, wx.EXPAND|wx.ALL, 5)

        v_sizer.AddSpacer((0, 0), 1, wx.EXPAND)
        v_sizer.Add(FilePicker(self, sticky_history=sticky_history), 0, wx.EXPAND)
        v_sizer.AddSpacer((0, 0), 1, wx.EXPAND)

        self.SetSizer(v_sizer)

    def file_picker_on_load(self, new_entry):
        new_path = os.path.dirname(new_entry) + '/'
        self.dir_picker.set_current_entry(new_path)
        self.dir_picker.prepend_to_history(new_path)
        set_textctrl_background_color(self.dir_picker.current_textctrl, 'ok')
        self.file_picker.set_current_entry(os.path.basename(new_entry))
        self.file_picker.set_assumed_prefix(new_path)

if __name__ == '__main__':
    app=wx.App(False)
    myFrame = MyFrame()
    myFrame.Show()
    app.MainLoop()




from __future__ import absolute_import
from ztv import ZTV
import wx
from wx.lib.pubsub import Publisher
import time

# intended to be run with, e.g.:
#     ipython
#     run launch_ztv.py
# or
#     ipython
#     from ztv_examples.fits_faker_panel.launch_ztv import launch_ztv
#     z = launch_ztv()

def launch_ztv():
    z = ZTV(control_panels_module_path='ztv_examples.fits_faker_panel.control_panels')
    z.start_fits_faker = lambda: z._send_to_ztv('fits-faker-start')
    z.stop_fits_faker = lambda: z._send_to_ztv('fits-faker-stop')
    z.control_panel('Faker')
    return z

if __name__ == '__main__':
    z = launch_ztv()


from __future__ import absolute_import
import wx
from .fake_fits_maker import FakeFitsMaker
from wx.lib.pubsub import Publisher

class FitsFakerPanel(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL)
        self.ztv_frame = self.GetTopLevelParent()
        v_sizer1 = wx.BoxSizer(wx.VERTICAL)
        v_sizer1.AddSpacer((0, 0), 1, wx.EXPAND)

        static_text = wx.StaticText(self, wx.ID_ANY, u"Fits Faker", wx.DefaultPosition, wx.DefaultSize, 
                                    wx.ALIGN_CENTER )
        static_text.Wrap( -1 )
        v_sizer1.Add(static_text, 0, wx.ALL|wx.ALIGN_CENTER|wx.ALIGN_CENTER_VERTICAL, 0)

        static_text = wx.StaticText(self, wx.ID_ANY, u"Example of a panel added-on to ztv", wx.DefaultPosition, 
                                    wx.DefaultSize, wx.ALIGN_CENTER )
        static_text.Wrap( -1 )
        v_sizer1.Add(static_text, 0, wx.ALL|wx.ALIGN_CENTER|wx.ALIGN_CENTER_VERTICAL, 0)

        self.launch_button = wx.Button(self, wx.ID_ANY, u"Launch Fake Fits Maker", wx.DefaultPosition, wx.DefaultSize, 0)
        v_sizer1.Add(self.launch_button, 0, wx.ALL|wx.ALIGN_CENTER|wx.ALIGN_CENTER_VERTICAL, 2)
        self.launch_button.Bind(wx.EVT_BUTTON, self.on_launch_button)

        self.halt_button = wx.Button(self, wx.ID_ANY, u"Halt Fake Fits Maker", wx.DefaultPosition, wx.DefaultSize, 0)
        v_sizer1.Add(self.halt_button, 0, wx.ALL|wx.ALIGN_CENTER|wx.ALIGN_CENTER_VERTICAL, 2)
        self.halt_button.Bind(wx.EVT_BUTTON, self.on_halt_button)
        self.halt_button.Disable()
        
        static_text = wx.StaticText(self, wx.ID_ANY, u"Note: clicking 'Halt' will delete files in /tmp/ for you", wx.DefaultPosition, 
                                    wx.DefaultSize, wx.ALIGN_CENTER )
        static_text.Wrap( -1 )
        v_sizer1.Add(static_text, 0, wx.ALL|wx.ALIGN_CENTER|wx.ALIGN_CENTER_VERTICAL, 0)
        static_text = wx.StaticText(self, wx.ID_ANY, u"If you don't click Halt, you may want to go clean up *.fits in /tmp/", 
                                    wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_CENTER )
        static_text.Wrap( -1 )
        v_sizer1.Add(static_text, 0, wx.ALL|wx.ALIGN_CENTER|wx.ALIGN_CENTER_VERTICAL, 0)

        v_sizer1.AddSpacer((0, 0), 1, wx.EXPAND)
        self.SetSizer(v_sizer1)
        Publisher().subscribe(self.on_launch_button, "fits-faker-start")
        Publisher().subscribe(self.on_halt_button, "fits-faker-stop")

    def on_launch_button(self, evt=None):
        self.fake_fits_maker = FakeFitsMaker(ztv_frame_pid=self.ztv_frame.ztv_frame_pid)
        self.fake_fits_maker.start()
        self.launch_button.Disable()
        self.halt_button.Enable()
        self.ztv_frame.source_panel.load_sky_frame('/tmp/sky_frame.fits', False)
        self.ztv_frame.source_panel.load_flat_frame('/tmp/flat_frame.fits', False)
        self.ztv_frame.source_panel.autoload_curfile_file_picker_on_load('/tmp/n*.fits')    

    def on_halt_button(self, evt=None):
        self.launch_button.Enable()
        self.halt_button.Disable()
        self.fake_fits_maker.keep_running = False


from __future__ import absolute_import
from .source_panel import SourcePanel
from .plot_panel import PlotPanel
from .phot_panel import PhotPanel
from .stats_panel import StatsPanel
from .color_panel import ColorPanel

# in adapting this file to your own use, you will most likely put it in some other directory.
# the above default imports will probably then need to be:
# from ztv.source_panel import SourcePanel
# from ztv.plot_panel import PlotPanel
# from ztv.phot_panel import PhotPanel
# from ztv.stats_panel import StatsPanel
# from ztv.color_panel import ColorPanel

# The requirements for a file that defines which control panels to show in ztv is:
# An iterator named `control_panels_to_load` 
# that returns tuples of a title string (to be displayed in GUI)
#                     and the class definition for the panel.

control_panels_to_load = [("Source", SourcePanel),
                          ("Color", ColorPanel),
                          ("Plot", PlotPanel),
                          ("Stats", StatsPanel),
                          ("Phot", PhotPanel)]



from __future__ import absolute_import
import wx
from wx.lib.pubsub import pub
import matplotlib
from matplotlib.figure import Figure
try:
    from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg
except IOError:
    # on some linux installations this import needs to be done twice as the first time raises an error:
    #   IOError: [Errno 2] No such file or directory: '/tmp/matplotlib-parallels/fontList.cache'
    from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg
from matplotlib.patches import PathPatch
from matplotlib.path import Path
import numpy as np
import sys


class PlotPlotPanel(wx.Panel):
    def __init__(self, parent, dpi=None, **kwargs):
        wx.Panel.__init__(self, parent, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, **kwargs)
        self.ztv_frame = self.GetTopLevelParent()
        self.figure = Figure(dpi=None, figsize=(1.,1.))
        self.axes = self.figure.add_subplot(111)
        self.canvas = FigureCanvasWxAgg(self, -1, self.figure)
        self.Bind(wx.EVT_SIZE, self._onSize)

    def _onSize(self, event):
        self._SetSize()

    def _SetSize(self):
        pixels = tuple(self.GetClientSize())
        self.SetSize(pixels)
        self.canvas.SetSize(pixels)
        self.figure.set_size_inches(float(pixels[0])/self.figure.get_dpi(), float(pixels[1])/self.figure.get_dpi())



class PlotPanel(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize)
        self.ztv_frame = self.GetTopLevelParent()
        self.primary_image_patch = None

        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.plot_panel = PlotPlotPanel(self)
        self.sizer.Add(self.plot_panel, 1, wx.LEFT | wx.TOP | wx.EXPAND)
        
        self.hideshow_button = wx.Button(self, wx.ID_ANY, u"Hide", wx.DefaultPosition, wx.DefaultSize, 0)
        self.sizer.Add(self.hideshow_button, 0, wx.ALL|wx.ALIGN_RIGHT|wx.ALIGN_BOTTOM, 2)
        self.hideshow_button.Bind(wx.EVT_BUTTON, self.on_hideshow_button)

        self.SetSizer(self.sizer)
        self.Fit()
        self.start_pt = wx.RealPoint(0., 0.)
        self.end_pt = wx.RealPoint(0., 0.)
        self.redraw()
        pub.subscribe(self.update_line_plot_points, 'update-line-plot-points')
        pub.subscribe(self.on_new_xy0, 'new-slice-plot-xy0')
        pub.subscribe(self.on_new_xy1, 'new-slice-plot-xy1')
        pub.subscribe(self.redraw, 'primary-xy-limits-changed')
        pub.subscribe(self.redraw, 'redraw-image')

    def update_line_plot_points(self, msg):
        xy0, xy1 = msg
        self.start_pt.x, self.start_pt.y = xy0[0], xy0[1]
        self.end_pt.x, self.end_pt.y = xy1[0], xy1[1]
        self.redraw_overplot_on_image()
        self.redraw()

    def on_new_xy0(self, msg):
        self.start_pt.x, self.start_pt.y = msg
        self.redraw_overplot_on_image()
        self.redraw()

    def on_new_xy1(self, msg):   
        self.end_pt.x, self.end_pt.y = msg
        self.redraw_overplot_on_image()
        self.redraw()

    def redraw_overplot_on_image(self):
        if self.primary_image_patch is not None:
            self.ztv_frame.primary_image_panel.axes.patches.remove(self.primary_image_patch)
        if self.start_pt == self.end_pt:
            path = Path([self.start_pt, self.start_pt + (0.5, 0.),
                         self.start_pt, self.start_pt + (-0.5, 0.), 
                         self.start_pt, self.start_pt + (0., 0.5), 
                         self.start_pt, self.start_pt + (0., -0.5), self.start_pt], 
                        [Path.MOVETO, Path.LINETO, Path.LINETO, Path.LINETO, Path.LINETO, 
                         Path.LINETO, Path.LINETO, Path.LINETO, Path.LINETO])
        else:
            path = Path([self.start_pt, self.end_pt], [Path.MOVETO, Path.LINETO])
        self.primary_image_patch = PathPatch(path, color='magenta', lw=1)
        self.ztv_frame.primary_image_panel.axes.add_patch(self.primary_image_patch)
        self.ztv_frame.primary_image_panel.figure.canvas.draw()
        self.hideshow_button.SetLabel(u"Hide")        

    def remove_overplot_on_image(self):
        if self.primary_image_patch is not None:
            self.ztv_frame.primary_image_panel.axes.patches.remove(self.primary_image_patch)
        self.ztv_frame.primary_image_panel.figure.canvas.draw()
        self.primary_image_patch = None
        self.hideshow_button.SetLabel(u"Show")

    def on_hideshow_button(self, event):
        if self.hideshow_button.GetLabel() == 'Hide':
            self.hideshow_button.SetLabel(u"Show")
            self.remove_overplot_on_image()
        else:
            self.hideshow_button.SetLabel(u"Hide")        
            self.redraw_overplot_on_image()
            
    def redraw(self, msg=None):
        if self.start_pt == self.end_pt:
            if self.ztv_frame.proc_image.ndim == 2:
                positions = np.array([-0.5, 0.5])
                im_values = np.array([self.ztv_frame.proc_image[self.start_pt.y, self.start_pt.x]] * 2)
                cur_im_num = 0
                cur_im_value = self.ztv_frame.proc_image[self.start_pt.y, self.start_pt.x]
            else:
                positions = np.array([np.arange(-0.5, self.ztv_frame.proc_image.shape[0] - 1), 
                                      np.arange(0.5, self.ztv_frame.proc_image.shape[0])]).transpose().ravel()
                im_values = np.array([self.ztv_frame.proc_image[:, self.start_pt.y, self.start_pt.x],
                                      self.ztv_frame.proc_image[:, self.start_pt.y, self.start_pt.x]]).transpose().ravel()
                cur_im_num = self.ztv_frame.cur_display_frame_num
                cur_im_value = self.ztv_frame.proc_image[cur_im_num, self.start_pt.y, self.start_pt.x]
            self.plot_panel.axes.clear()
            self.line_plot = self.plot_panel.axes.plot(positions, im_values)
            self.plot_panel.axes.plot([cur_im_num], [cur_im_value], 'xm')
            self.plot_panel.axes.set_xlim([positions[0], positions[-1]])
            self.plot_panel.figure.canvas.draw()
        else:
            xlim = self.ztv_frame.primary_image_panel.xlim
            ylim = self.ztv_frame.primary_image_panel.ylim
            oversample_factor = 10.
            n_pts = oversample_factor*np.max(self.ztv_frame.display_image.shape)
            xs = np.linspace(self.start_pt.x, self.end_pt.x, n_pts)
            ys = np.linspace(self.start_pt.y, self.end_pt.y, n_pts)
            mask = ((xs >= min(xlim)) & (xs <= max(xlim)) & 
                    (ys >= min(ylim)) & (ys <= max(ylim)) &
                    (xs >= 0.) & (ys >= 0.) &
                    (xs < self.ztv_frame.display_image.shape[1]) & 
                    (ys < self.ztv_frame.display_image.shape[0]))
            xs = xs[mask]
            ys = ys[mask]
            if len(xs) > 0:
                if (ys.max() - ys.min()) > (xs.max() - xs.min()):   # dominantly a vertical slice
                    if ys[0] > ys[1]:
                        xs = xs[-1::-1]
                        ys = ys[-1::-1]
                else:
                    if xs[0] > xs[1]:
                        xs = xs[-1::-1]
                        ys = ys[-1::-1]
                if np.min(np.round(xs)) == np.max(np.round(xs)):
                    positions = ys
                elif np.min(np.round(ys)) == np.max(np.round(ys)):
                    positions = xs
                else:
                    positions = np.sqrt( (xs - xs[0])**2 + (ys - ys[0])**2 )
                xs = xs.astype(np.int)
                ys = ys.astype(np.int)
                im_values = self.ztv_frame.display_image[ys, xs]
                self.plot_panel.axes.clear()
                if positions.min() != positions.max():
                    self.line_plot = self.plot_panel.axes.plot(positions, im_values)
                    self.plot_panel.axes.set_xlim([positions[0], positions[-1]])
                self.plot_panel.figure.canvas.draw()
            else:
                self.plot_panel.axes.clear()
                self.plot_panel.figure.canvas.draw()
        


import wx

textctrl_output_only_background_color = (235, 235, 235)
        
def set_textctrl_background_color(textctrl, mode, tooltip=None):
    if mode == 'ok':
        color = (255,255,255)
    elif mode == 'enter-needed':
        color = (200,255,200)
    elif mode == 'invalid':
        color = (255,200,200)
    textctrl.SetBackgroundColour(color)
    textctrl.Refresh()
    if tooltip is not None and not isinstance(tooltip, wx.ToolTip):
        tooltip = wx.ToolTip(tooltip)
    textctrl.SetToolTip(tooltip)
    force_textctrl_color_update(textctrl)

def force_textctrl_color_update(textctrl):
    cur_focused_item = textctrl.GetParent().FindFocus()
    insertion_point = textctrl.GetInsertionPoint()
    children = textctrl.GetParent().GetChildren()
    can_accept_focus_mask = [(hasattr(a, 'CanAcceptFocus') and a.CanAcceptFocus() and 
                              (a is not textctrl)) for a in children]
    if True in can_accept_focus_mask:
        children[can_accept_focus_mask.index(True)].SetFocus()
    textctrl.SetFocus()
    textctrl.SetInsertionPoint(insertion_point)
    if cur_focused_item is not None:
        cur_focused_item.SetFocus()

def validate_textctrl_str(textctrl, validate_fxn, last_value):
    """
    can accept arbitrary functions in validate_fxn.  They just need to raise a ValueError if
    they don't like the input.
    """
    try:
        newval = validate_fxn(textctrl.GetValue())
        if textctrl.GetValue() == last_value:
            set_textctrl_background_color(textctrl, 'ok')
        else:
            set_textctrl_background_color(textctrl, 'enter-needed',
                                         'Press enter in this field to set new minimum value')
        return True
    except ValueError:
        # TODO: figure out some (clever?) way of having validate_fxn give info about itself that is more useful in the following error tooltip message
        set_textctrl_background_color(textctrl, 'invalid', 
                                      'Entry cannot be converted to {}'.format(str(validate_fxn)))
        return False



import sys
import pickle
from threading import Thread
from Queue import Queue, Empty


# point is to make improbable that would ever happen to appear inside a pickled image and be mistaken
# Note that because you may be loading image data and random strings will occur... best to have a big
# *10 on this message string.  *1 or *2 might not be enough....
end_of_message_message = ("---EndOfMessage---"*10) + "\n"   

def send_to_stream(stream, msg): 
    """
    Pickle & send to stdout a message.
    Used primarily to communicate back-and-forth with a separately launched ztv process.
    """
    if isinstance(msg, str):
        msg = (msg,)
    pkl = pickle.dumps(msg)
    stream.write(pkl + '\n' + end_of_message_message)
    stream.flush()

class UnexpectedEndOfStream(Exception): pass

class StreamListenerTimeOut(Exception): pass

def _accumulate_to_queue(stream, queue):
    while True:
        line = stream.readline()
        if line:
            queue.put(line)
        else:
# TODO: rather than return, should really raise the Error, but then code elsewhere needs to be catching for it
#             raise UnexpectedEndOfStream
            return

class StreamListener():
    def __init__(self, stream):
        """
        stream: e.g, stdin/stdout
        """
        self.stream = stream
        self.queue = Queue()
        self.thread = Thread(target=_accumulate_to_queue, args=(self.stream,self.queue))
        self.thread.daemon = True
        self.thread.start() 

    def read_pickled_message(self, timeout=None):
        try:
            block = timeout is not None
            msg = ""
            while not msg.endswith('\n' + end_of_message_message):
                msg += self.queue.get(block=block, timeout=timeout)
            return pickle.loads(msg.replace('\n' + end_of_message_message, ''))
        except Empty:
            raise StreamListenerTimeOut


def listen_to_pipe(pipe, timeout=None):
    """
    Will listen on pipe until has seen end_of_message_message, then strip the 
    end_of_message_message and return the unpickled version of the preceding message
    
    If timeout is None, then will just do a traditional blocking call of readline() 
                        (i.e. will *never* return if a newline never comes in)
    If timeout is not None, then if length of output hasn't changed in timeout seconds,
                        then raise a timeout exception
    """
    in_str = ""
    if timeout is None:
        while not in_str.endswith('\n' + end_of_message_message):
            in_str += pipe.readline()
    else:
        pass
    return pickle.loads(in_str.replace('\n' + end_of_message_message, ''))


from setuptools import setup, find_packages  # Always prefer setuptools over distutils
from codecs import open  # To use a consistent encoding
import os

base_dir = os.path.abspath(os.path.dirname(__file__))

about = {}
with open(os.path.join(base_dir, "ztv", "__about__.py")) as f:
    exec(f.read(), about)

# Get the long description from the relevant file, converting from md to rst if possible
with open(os.path.join(base_dir, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()
try:
    from pypandoc import convert
    long_description = convert('README.md', 'rst', format='md')
    # also, convert screen shot links to point to github
    import re
    rep = re.compile('screenshots/(.*).png')
    long_description = rep.sub(r'https://raw.githubusercontent.com/henryroe/ztv/master/screenshots/\1.png', long_description)
except ImportError:
    print("warning: pypandoc module not found, could not convert Markdown to RST")


setup(
    name=about["__title__"],

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=about["__version__"],

    description=about["__summary__"],
    long_description=long_description,

    # The project's main homepage.
    url=about["__uri__"],

    # Author details
    author=about["__author__"],
    author_email=about["__email__"],

    # Choose your license
    license=about["__license__"],

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #     Development Status :: 1 - Planning
        #     Development Status :: 2 - Pre-Alpha
        #     Development Status :: 3 - Alpha
        #     Development Status :: 4 - Beta
        #     Development Status :: 5 - Production/Stable
        #     Development Status :: 6 - Mature
        #     Development Status :: 7 - Inactive
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Intended Audience :: Science/Research',
        'Topic :: Multimedia :: Graphics :: Viewers',
        'Topic :: Scientific/Engineering :: Astronomy',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
#         'Programming Language :: Python :: 2.6',  # no reason to believe won't work on 2.6, but am not testing
        'Programming Language :: Python :: 2.7',
#  Intention is to write to v3 compatibility and eventually test, but am not doing that as of now (2015-03-06)
#         'Programming Language :: Python :: 3',
#         'Programming Language :: Python :: 3.2',
#         'Programming Language :: Python :: 3.3',
#         'Programming Language :: Python :: 3.4',
    ],

    # What does your project relate to?
    keywords='astronomy image viewer fits',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),

    # List run-time dependencies here.  These will be installed by pip when your
    # project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    # Note: stompy is needed for ActiveMQ integration, but can be safely ignored unless needed
    # Note: astropy will require numpy, so no need to specify here (had been: 'numpy>=1.8.1')
    # Note: matplotlib version requirement could maybe be even earlier as we're not doing anything bleeding edge.
    # Note: scipy not required, but highly recommended and some functionality may be lost without it. 
    #       (at time of writing, v0.1.0, you lost some of the analysis in the phot_panel.py)
    install_requires=['astropy>=1.0.0', 'wxPython>=3.0', 'matplotlib>=1.3', 'psutil>=2.0', 'astropy_helpers>=1.0.0'],

    # List additional groups of dependencies here (e.g. development dependencies).
    # You can install these using the following syntax, for example:
    # $ pip install -e .[dev,test]
# 2015-03-06: haven't considered if we need/want any of these
#     extras_require = {
#         'dev': ['check-manifest'],
#         'test': ['coverage'],
#     },

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
# 2015-03-06: haven't considered if we need/want any of these
#     package_data={
#         'sample': ['package_data.dat'],
#     },

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages.
    # see http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
# 2015-03-06: haven't considered if we need/want any of these
#     data_files=[('my_data', ['data/data_file'])],

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
# 2015-03-06: haven't considered if we need/want any of these
#     entry_points={
#         'console_scripts': [
#             'sample=sample:main',
#         ],
#     },
)



import numpy as np
from astropy.io import fits
import time
import os.path
import os
import psutil
import threading


class FakeFitsMaker(threading.Thread):
    def __init__(self, ztv_frame_pid=None):
        self.ztv_frame_pid = ztv_frame_pid  # will kill self if this pid no longer alive
        self.nx = 512
        self.ny = 512
        self.flat_field_pixel_to_pixel_fractional_1sigma = 0.15
        self.flat_field_num_dust_donuts = 30
        self.flat_field_dust_donuts_radius_range = [20, 50]
        self.flat_field_dust_donuts_width_range = [10, 15]
        self.flat_field_dust_donuts_peak_range = [0.5, 0.8]
        self.sky_pattern_mean_cts = 9000.
        self.sky_pattern_row_to_row_variation_1sigma_cts = 2000.
        self.saturation_cts = 2**16
        self.seeing_gauss_width = 2.0  # not fwhm....being lazy
        self.n_bkgd_stars = 50
        
        self.n_moving_objects = 10
        self.moving_objects = []
        
        self.delay_between_frames_sec = 2.0
        
        self.data_dir = '/tmp/'
        self.files_to_delete = []
        self.frame_number = 1
        
        self.set_up_bkgd_stars()
        self.make_flat_frame()
        self.make_sky_frame()
        self.write_to_fits_file(self.sky_frame, 'sky_frame.fits')
        self.write_to_fits_file(self.flat_frame, 'flat_frame.fits')
        threading.Thread.__init__(self)
        self.daemon = True

    def run(self):
        self.keep_running = True
        while self.keep_running:
            im = self.make_data_frame()
            self.write_to_fits_file(im, 'current.fits')
            self.write_to_fits_file(im, 'n{:04d}.fits'.format(self.frame_number))
            self.frame_number += 1
            time.sleep(self.delay_between_frames_sec)
            if not psutil.pid_exists(self.ztv_frame_pid):
                self.keep_running = False
        self.delete_files()
    
    def set_up_bkgd_stars(self):
        flux_lognormal_sigma = 1.0
        flux_multiplier = 7000.
        self.bkgd_stars = {}
        self.bkgd_stars['x'] = np.random.uniform(low=0., high=self.nx - 1, size=self.n_bkgd_stars)
        self.bkgd_stars['y'] = np.random.uniform(low=0., high=self.ny - 1, size=self.n_bkgd_stars)
        self.bkgd_stars['peak_cts'] = flux_multiplier * np.random.lognormal(sigma=flux_lognormal_sigma, 
                                                                            size=self.n_bkgd_stars)
        self.bkgd_stars_frame = np.zeros([self.ny, self.nx])
        for i in np.arange(self.n_bkgd_stars):
            dxs = np.outer(np.ones(self.ny), np.arange(self.nx)) - self.bkgd_stars['x'][i]
            dys = np.outer(np.arange(self.ny), np.ones(self.nx)) - self.bkgd_stars['y'][i]
            self.bkgd_stars_frame += (self.bkgd_stars['peak_cts'][i] *
                                      np.exp(-((dxs)**2 + (dys)**2) / (2. * self.seeing_gauss_width**2)))

    def new_moving_object(self):
        flux_lognormal_sigma = 1.0
        flux_multiplier = 10000.
        new_object = {'peak_cts': flux_multiplier * np.random.lognormal(sigma=flux_lognormal_sigma)}
        new_object['x'] = 0.
        new_object['y'] = np.random.uniform(low=1., high=self.ny - 2)
        new_object['dx'] = np.random.normal(loc=20, scale=15.)
        new_object['dy'] = np.random.normal(loc=0., scale=10.)
        return new_object
        
    def advance_moving_objects(self):
        remaining_moving_objects = []
        for cur_moving_object in self.moving_objects:
            cur_moving_object['x'] += cur_moving_object['dx']
            cur_moving_object['y'] += cur_moving_object['dy']
            if ((cur_moving_object['x'] >= 0.) and (cur_moving_object['y'] >= 0.) and
                (cur_moving_object['x'] < self.nx) and (cur_moving_object['y'] < self.ny)):
                remaining_moving_objects.append(cur_moving_object)
        self.moving_objects = remaining_moving_objects

    def calc_one_sky(self):
        return np.array([np.random.poisson(a, size=self.nx) for a in self.sky_frame_row_baseline])
        
    def make_sky_frame(self):
        self.sky_frame_row_baseline = np.random.normal(loc=self.sky_pattern_mean_cts, scale=
                                                       self.sky_pattern_row_to_row_variation_1sigma_cts,
                                                       size=self.ny)
        self.sky_frame_row_baseline[self.sky_frame_row_baseline <= 0] = 0.
        self.sky_frame = self.calc_one_sky() * self.flat_frame
    
    def make_flat_frame(self):
        flat = np.random.normal(loc=1.0, scale=self.flat_field_pixel_to_pixel_fractional_1sigma, size=[self.ny, self.nx])
        for i in range(self.flat_field_num_dust_donuts):
            x = np.random.uniform(0., self.nx)
            y = np.random.uniform(0., self.ny)
            radius = np.random.uniform(min(self.flat_field_dust_donuts_radius_range), 
                                       max(self.flat_field_dust_donuts_radius_range))
            width = np.random.uniform(min(self.flat_field_dust_donuts_width_range), 
                                      max(self.flat_field_dust_donuts_width_range))
            peak = np.random.uniform(min(self.flat_field_dust_donuts_peak_range),
                                     max(self.flat_field_dust_donuts_peak_range))
            xdist = np.outer(np.ones(flat.shape[0]), np.arange(flat.shape[1]) - x)
            ydist = np.outer(np.arange(flat.shape[0]) - y, np.ones(flat.shape[1]))
            dist = np.sqrt(xdist**2 + ydist**2)
            flat *= 1. - (1. - peak) * np.exp(-((dist - radius)**2) / (2. * width**2))
        self.flat_frame = flat
                                           
    def make_data_frame(self):
        im = self.bkgd_stars_frame.copy()
        self.advance_moving_objects()
        while len(self.moving_objects) < self.n_moving_objects:
            self.moving_objects.append(self.new_moving_object())
        for cur_moving_object in self.moving_objects:
            dxs = np.outer(np.ones(self.ny), np.arange(self.nx)) - cur_moving_object['x']
            dys = np.outer(np.arange(self.ny), np.ones(self.nx)) - cur_moving_object['y']
            im += (cur_moving_object['peak_cts'] * np.exp(-((dxs)**2 + (dys)**2) / (2. * self.seeing_gauss_width**2)))
        im = (im + self.calc_one_sky()) * self.flat_frame
        for x in np.arange(self.nx):  # has to be a better way than this dumb/slow loop
            for y in np.arange(self.ny):
                im[y, x] = min(np.random.poisson(max(im[y, x], 0)), self.saturation_cts)
        return im

    def delete_files(self):
        for curfile in self.files_to_delete:
            try:
                os.remove(os.path.join(self.data_dir, curfile))
            except OSError:  # ignore error if file doesn't exist
                pass

    def write_to_fits_file(self, im, filename):
        max_files_on_disk = 10  # play nice with space in people's /tmp/ dirs
        hdu = fits.PrimaryHDU(im)
        hdu.writeto(os.path.join(self.data_dir, filename), clobber=True)
        if filename not in self.files_to_delete:
            if filename.startswith('n'):
                self.files_to_delete = ([a for a in self.files_to_delete if a.startswith('n')] + 
                                        [filename] +
                                        [a for a in self.files_to_delete if not a.startswith('n')])
            else:
                self.files_to_delete.append(filename)
        while len(self.files_to_delete) > max_files_on_disk:
            os.remove(os.path.join(self.data_dir, self.files_to_delete.pop(0)))
            

if __name__ == '__main__':
    f = FakeFitsMaker()
    f.start()


import numpy as np
from astropy.stats import sigma_clipped_stats
import sys

class Error(Exception):
    pass

def centroid(im, x0, y0, searchboxsize=5, centroidboxsize=9):
    """
    im - 2-d numpy array
    x0, y0 - initial x,y
    searchboxsize - search for peak value within this box
    centroidboxsize - calculate centroid of this size box around maximum
            (have only thought through these size parameters when they are ODD numbers. may not behave nicely if even.)
    
    returns x,y
    ---
    Note: minimal error-checking as intend to replace this some day with astropy photometry package once it is stable and released
    """
    xmax0 = int(np.round(x0 - searchboxsize/2. + 0.5))
    ymax0 = int(np.round(y0 - searchboxsize/2. + 0.5))
    # in case of multiple maxima, want mean position
    subim = im[ymax0:ymax0 + searchboxsize, xmax0:xmax0 + searchboxsize]
    if subim.size < 9:
        return np.nan, np.nan
    ymax, xmax = [b.mean() for b in np.where(subim == subim.max())]
    xmax = int(np.round(xmax + xmax0 - centroidboxsize/2.))
    ymax = int(np.round(ymax + ymax0 - centroidboxsize/2.))
    subim = im[ymax:ymax + centroidboxsize, xmax:xmax + centroidboxsize]
    subim_x = np.outer(np.ones(subim.shape[0]), np.arange(subim.shape[1]) + xmax)
    subim_y = np.outer(np.arange(subim.shape[0]) + ymax, np.ones(subim.shape[1]))
    return (subim_x*subim).sum()/subim.sum(), (subim_y*subim).sum()/subim.sum()


def aperture_phot(im, x, y, star_radius, sky_inner_radius, sky_outer_radius, 
                  return_distances=False):
    """
    im - 2-d numpy array
    x,y - coordinates of center of star
    star_radius - radius of photometry circle
    sky_inner_radius, sky_outer_radius - defines annulus for determining sky
            (if sky_inner_radius > sky_outer_radius, aperture_phot flips them)
    ----
    Note that this is a very quick-and-dirty aperture photometry routine.
    No error checking.
    No partial pixels.
    Many ways this could fail and/or give misleading results.
    Not to be used within 12 hours of eating food.
    Use only immediately after a large meal.
    ----
    returns dictionary with:
    flux - sky-subtracted flux inside star_radius
    sky_per_pixel - sky counts per pixel determined from sky annulus
    sky_per_pixel_err - estimated 1-sigma uncertainty in sky_per_pixel
    sky_err - estimated 1-sigma uncertainty in sky subtraction from flux
    n_star_pix - number of pixels in star_radius
    n_sky_pix - number of pixels in sky annulus
    x - input x
    y - input y
    star_radius - input star_radius
    sky_inner_radius - input sky_inner_radius
    sky_outer_radius - input sky_outer_radius 
    """
    if np.isnan(x) or np.isnan(y):
        return {'error-msg':'One or both of x/y were NaN.', 'x':x, 'y':y, 'star_radius': star_radius,
                'sky_inner_radius': sky_inner_radius, 'sky_outer_radius': sky_outer_radius,
                'n_star_pix':0, 'n_sky_pix':0, 'sky_per_pixel':np.nan, 'sky_per_pixel_err':np.nan,
                'flux':np.nan, 'sky_err':np.nan, 'distances':[]}
    if sky_inner_radius > sky_outer_radius:
        sky_inner_radius, sky_outer_radius = sky_outer_radius, sky_inner_radius
    output = {'x': x, 'y': y, 'star_radius': star_radius,
              'sky_inner_radius': sky_inner_radius, 'sky_outer_radius': sky_outer_radius}
    xdist = np.outer(np.ones(im.shape[0]), np.arange(im.shape[1]) - x)
    ydist = np.outer(np.arange(im.shape[0]) - y, np.ones(im.shape[1]))
    dist = np.sqrt(xdist**2 + ydist**2)
    star_mask = dist <= star_radius
    star_pixels = im[star_mask]
    
    sky_pixels = im[(dist >= sky_inner_radius) & (dist <= sky_outer_radius)]
    output['n_star_pix'] = star_pixels.size
    output['n_sky_pix'] = sky_pixels.size
    sky_per_pixel, median, stddev = sigma_clipped_stats(sky_pixels)
    sky_per_pixel_err = stddev/np.sqrt(sky_pixels.size)
    output['sky_per_pixel'] = sky_per_pixel
    # TODO: check that are doing sky_per_pixel_err right.  In one quick test seemed high (but maybe wasn't a good test)
    output['sky_per_pixel_err'] = sky_per_pixel_err
    output['flux'] = star_pixels.sum() - sky_per_pixel*star_pixels.size
    output['sky_err'] = sky_per_pixel_err*np.sqrt(star_pixels.size)
    if return_distances:
        output['distances'] = dist
    return output